import 'package:bdlive_flutter_viewer/bdlive_flutter_viewer.dart';
import 'package:flutter/material.dart';

import 'common_radio_form_item.dart';
import 'product_detail_page.dart';

class SettingActivityPage extends StatefulWidget {
  const SettingActivityPage({super.key});

  @override
  State<SettingActivityPage> createState() => _SettingActivityPageState();
}

class _SettingActivityPageState extends State<SettingActivityPage> {
  final TextEditingController _activityIdController =
      TextEditingController(text: "1812562329965659");
  final TextEditingController _tokenController = TextEditingController(
      text: "CWwVfquL9pP9A5VcwuE+32y4hTaQHrNwFkpdOAnW/wk=");
  var _isLoading = false;

  BdlViewerAuthMode _authMode = BdlViewerAuthMode.custom;
  bool _isPortrait = true;
  int _openUrlType = 1;

  void showCardPage(BdlActivityConfig activity, String url,
      BdlViewerOpenUrlType type, bool enableFloating) {
    Navigator.of(context).push(MaterialPageRoute(builder: (ctx) {
      return ProductDetailPage(
        activity: activity,
        url: url,
        type: type,
        enableFloating: enableFloating,
        isShowBackArrow: false,
      );
    }));
    ProductDetailPage.pageIsExist = true;

    BdliveViewerApi.instance.manualClickLiveRoomExitBtn();
  }

  void enterLiveRoom() async {
    setState(() {
      _isLoading = true;
    });

    var activityId = int.parse(_activityIdController.text);
    var token = _tokenController.text;
    BdlActivityConfig activity = BdlActivityConfig(
        activityId: activityId,
        token: token,
        authMode: _authMode,
        isPortrait: _isPortrait);

    var isPreLoadSuccess =
        await BdliveViewerApi.instance.enterLiveRoom(activity);
    setState(() {
      _isLoading = false;
    });
    if (isPreLoadSuccess) {
      BdlLiveRoomStatusListener listener = BdlLiveRoomStatusListener(
        onLiveRoomActivityResume: (activity) {
          debugPrint('onLiveRoomActivityResume...$activity');
          if (ProductDetailPage.pageIsExist) {
            Navigator.of(context).pop();
            ProductDetailPage.pageIsExist = false;
          }
        },
        onLiveRoomActivityPause: (activity) {
          debugPrint('onLiveRoomActivityPause...$activity');
        },
        onLiveRoomFullDestroyed: (activity) {
          debugPrint('onLiveRoomFullDestroyed...$activity');
          BdliveViewerApi.instance.setLiveRoomStatusListener(null);
        },
        onFloatViewWillAppear: (activity) {
          debugPrint('onFloatViewWillAppear...$activity');
        },
        onFloatViewWillClose: (activity) {
          debugPrint('onFloatViewWillClose...$activity');
        },
        onOpenUrlCallback: (url, type, enableFloating) {
          debugPrint('onOpenUrlCallback $url');
          debugPrint('onOpenUrlCallback $_openUrlType');
          if (_openUrlType == 1) {
            // Open new page
            showCardPage(activity, url, type, enableFloating);
          } else {
            // SDK default opening
            BdliveViewerApi.instance.openUrl(url, type, enableFloating);
          }
        },
      );
      BdliveViewerApi.instance.setLiveRoomStatusListener(listener);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Setting Activity'),
      ),
      body: ListView(
        padding: EdgeInsets.all(10),
        children: [
          TextField(
            controller: _activityIdController,
            keyboardType: TextInputType.number,
            decoration: InputDecoration(
                hintText: 'Please enter activityId',
                hintStyle: TextStyle(fontSize: 16, color: Colors.grey),
                suffixIcon: IconButton(
                    onPressed: () {
                      _activityIdController.clear();
                    },
                    icon: Icon(Icons.clear))),
          ),
          const SizedBox(
            height: 10,
          ),
          TextField(
            controller: _tokenController,
            decoration: InputDecoration(
                hintText: 'Please enter token',
                hintStyle: TextStyle(fontSize: 16, color: Colors.grey),
                suffixIcon: IconButton(
                    onPressed: () {
                      _tokenController.clear();
                    },
                    icon: Icon(Icons.clear))),
          ),
          const SizedBox(
            height: 10,
          ),
          CommonRadioFormItem(
            'authMode',
            options: const ['custom', 'public'],
            values: const [
              BdlViewerAuthMode.custom,
              BdlViewerAuthMode.public,
            ],
            groupValue: _authMode,
            onChanged: (val) {
              setState(() {
                _authMode = val!;
              });
            },
          ),
          CommonRadioFormItem('Landscape or Portrait',
              options: ['Landscape', 'Portrait'],
              values: [false, true],
              groupValue: _isPortrait, onChanged: (val) {
            setState(() {
              _isPortrait = val!;
            });
          }),
          const SizedBox(
            height: 30,
          ),
          ElevatedButton(
            onPressed: () {
              if (_activityIdController.text.trim().isEmpty) {
                print('activityId cannot be empty');
                return;
              }
              if (_tokenController.text.trim().isEmpty) {
                print('token cannot be empty');
                return;
              }
              // Hide keyboard
              FocusScope.of(context).unfocus();

              enterLiveRoom();
            },
            child: Text(_isLoading ? 'Joining...' : 'Enter Live Room'),
          ),
          const SizedBox(
            height: 20,
          ),
          ElevatedButton(
              onPressed: () {
                debugPrint("manualClickLiveRoomExitBtn--");
                if (_activityIdController.text.trim().isEmpty) {
                  debugPrint('activityId cannot be empty');
                  return;
                }
                if (_tokenController.text.trim().isEmpty) {
                  debugPrint('token cannot be empty');
                  return;
                }
                // Hide keyboard
                FocusScope.of(context).unfocus();

                enterLiveRoom();

                Future.delayed(Duration(seconds: 6), () {
                  BdliveViewerApi.instance.manualClickLiveRoomExitBtn();
                  print('Time is up');
                });
              },
              child: const Text(
                  "Simulate leaving live room, executes after 10s",
                  style: TextStyle(color: Colors.green, fontSize: 22))),
          const SizedBox(
            height: 10,
          ),
          CommonRadioFormItem(
            'openUrl Method',
            options: const ['Open new page', ' SDK default open'],
            values: const [1, 2],
            groupValue: _openUrlType,
            onChanged: (val) {
              setState(() {
                _openUrlType = val!;
              });
            },
          ),
        ],
      ),
    );
  }
}
