#import "BdliveFlutterViewerPlugin.h"
#import <objc/message.h>
#import <TTSDKFramework/TTSDKFramework.h>
#import <BDLive/BDLive.h>
#import "BdliveFlutterViewerDelegate.h"
#import "BdliveFlutterViewerPlayer.h"

@interface BDLEnv : NSObject

@end

@interface BDLEnv ()

+ (void)setChannelType:(BDLChannelType)type;

@end

@interface BdliveFlutterViewerPlugin ()

@property (nonatomic, weak) NSObject<FlutterPluginRegistrar> *registrar;
@property (nonatomic, strong) BdliveFlutterViewerDelegate *delegate;
@property (nonatomic, strong) NSMutableDictionary *playerMap;

@property (nonatomic, strong) BDLActivity *activity;

@end

@implementation BdliveFlutterViewerPlugin

+ (instancetype)sharedInstance {
    static BdliveFlutterViewerPlugin *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[self alloc] init];
        if ([BDLEnv respondsToSelector:@selector(setChannelType:)]) {
            [BDLEnv setChannelType:BDLChannelTypeFlutter];
        }
    });
    return instance;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    BdliveFlutterViewerPlugin *plugin = [self sharedInstance];
    plugin.registrar = registrar;
    
    FlutterMethodChannel *channel = [FlutterMethodChannel
                                     methodChannelWithName:@"com.bytedance.flutter_viewer.plugin"
                                     binaryMessenger:[registrar messenger]];
    [registrar addMethodCallDelegate:plugin channel:channel];
    
    BdliveViewerViewFactory *fc = [[BdliveViewerViewFactory alloc] init];
    [registrar registerViewFactory:fc withId:@"BdliveViewerPlayerView"];
    plugin.viewerViewFactory = fc;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    SEL selector = NSSelectorFromString([call.method stringByAppendingString:@":result:"]);
    if ([self respondsToSelector:selector]) {
        ((void(*)(id,SEL,id,FlutterResult))objc_msgSend)(self, selector, call.arguments, result);
        return;
    }
    else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)startWithConfiguration:(NSDictionary *)arguments result:(FlutterResult)result {
    NSString *appID = arguments[@"appID"];
    NSString *bundleID = arguments[@"bundleID"];
    NSString *appName = arguments[@"appName"];
    NSString *channel = arguments[@"channel"];
    NSString *appVersion = arguments[@"appVersion"];
    NSString *liveLicenseFilePath = arguments[@"liveLicenseFilePath"];
    NSString *vodLicenseFilePath = arguments[@"vodLicenseFilePath"];
    
    TTSDKConfiguration *configuration = [TTSDKConfiguration defaultConfigurationWithAppID:appID];
    if (bundleID) {
        configuration.bundleID = bundleID;
    }
    if (appName) {
        configuration.appName = appName;
    }
    if (channel) {
        configuration.channel = channel;
    }
    if (appVersion) {
        configuration.appVersion = appVersion;
    }
    
    NSString *key = [self.registrar lookupKeyForAsset:vodLicenseFilePath];
    NSString *path = [[NSBundle mainBundle] pathForResource:key ofType:nil];
    configuration.bizType = TTSDKServiceBizType_All & ~TTSDKServiceBizType_Live;
    configuration.licenseFilePath = path;
    [TTSDKManager startWithConfiguration:configuration];
    
    key = [self.registrar lookupKeyForAsset:liveLicenseFilePath];
    path = [[NSBundle mainBundle] pathForResource:key ofType:nil];
    configuration.bizType = TTSDKServiceBizType_Live;
    configuration.licenseFilePath = path;
    [TTSDKManager startWithConfiguration:configuration];
    
    result(nil);
}

- (void)enterLiveRoom:(NSDictionary *)arguments result:(FlutterResult)result {
    BDLActivity *activity = [[BDLActivity alloc] init];
    
    activity.activityId = arguments[@"activityId"];
    activity.token = arguments[@"token"];
    activity.authMode = [arguments[@"authMode"] integerValue];
    activity.isPortrait = [arguments[@"isPortrait"] boolValue];
    
    BDLLiveEngine *engine = [BDLLiveEngine sharedInstance];
    BDLLivePullViewController *livePullVC = [engine getLivePullViewController];
    if ([self.activity isEqual:activity] && livePullVC) { // No more authentication is required. The live room is displayed directly.
        if (livePullVC.isFloating) {
            [livePullVC hideFloatingPlayerIfAvailable:YES];
        }
        else {
            [self configLivePullVC:livePullVC];
            [self.delegate showLivePullViewController:livePullVC];
        }
        result(@(YES));
        return;
    }
    
    [engine joinLiveRoomWithActivity:activity success:^{
        self.activity = activity;
        
        BDLLivePullViewController *livePullVC = [engine getLivePullViewController];
        [self configLivePullVC:livePullVC];
        [self.delegate showLivePullViewController:livePullVC];
        
        result(@(YES));
    } failure:^(NSError * _Nonnull error) {
        result(@(NO));
    }];
}

- (void)configLivePullVC:(BDLLivePullViewController *)livePullVC {
    BdliveFlutterViewerDelegate *delegate = [BdliveFlutterViewerDelegate eventHandlerWithName:@"com.bytedance.flutter_viewer.listener.event" binaryMessenger:[self.registrar messenger]];
    self.delegate = delegate;
    delegate.activity = self.activity;
    [livePullVC setDelegate:delegate];
    livePullVC.actionProvider = delegate;
    
    BDLPlayerCommonConfiguration *config = [[BDLLiveEngine sharedInstance] liveRoomConfiguration].playerConfig.common;
    config.enableVodPiP = NO;
    config.enableLivePiP = NO;
    
    __weak typeof(self) weakSelf = self;
    livePullVC.config.customShoppingCardController = ^(BDLLivePullViewController * _Nonnull viewController, BDLShoppingCardController * _Nonnull shoppingCardController) {
        shoppingCardController.openUrlBlock = ^(BDLShoppingCardController * _Nonnull controller, NSURL * _Nonnull url, BOOL isFloatingEnable) {
            __strong typeof(weakSelf) self = weakSelf;
            [self.delegate openUrl:[url absoluteString] enableFloating:isFloatingEnable type:BDLPageAdvertisementTypeCardAd];
        };
    };

    livePullVC.config.customizePageAdView = ^UIView<BDLPageAdViewProtocol> * _Nullable(BDLLivePullViewController * _Nonnull viewController, BDLPageAdView * _Nonnull view) {
        view.viewTappedBlock = ^(BDLPageAdView * _Nonnull view, BDLAdvertisementModel * _Nonnull ad, BOOL isFloatingEnable) {
            __strong typeof(weakSelf) self = weakSelf;
            [self.delegate openUrl:ad.advertisementRedirectUrl enableFloating:isFloatingEnable type:ad.pageAdvertisementType];
        };
        return view;
    };
    
    livePullVC.config.customFloatingController = ^(BDLLivePullViewController * _Nonnull viewController, BDLFloatingController * _Nonnull floatingController) {
        floatingController.openUrlBlock = ^(BDLFloatingController * _Nonnull floatingController, NSURL * _Nonnull url, BOOL isFloatingEnable) {
            __strong typeof(weakSelf) self = weakSelf;
            [self.delegate openUrl:[url absoluteString] enableFloating:isFloatingEnable type:BDLPageAdvertisementTypeFloatingAd];
        };
    };
    livePullVC.floatingPlayerViewWillAppear = ^(BDLLivePullViewController * _Nonnull viewController, BDLFloatingPlayer * _Nonnull floatingPlayer) {
        __strong typeof(weakSelf) self = weakSelf;
        [self.delegate emitEvent:@{
            @"activity" : [self.activity yy_modelToJSONObject]
        } methodName:@"onFloatViewWillAppear"];
    };
    livePullVC.floatingPlayerViewWillClose = ^(BDLLivePullViewController * _Nonnull viewController, BDLFloatingPlayer * _Nonnull floatingPlayer) {
        __strong typeof(weakSelf) self = weakSelf;
        [self.delegate emitEvent:@{
            @"activity" : [self.activity yy_modelToJSONObject]
        } methodName:@"onFloatViewWillClose"];
    };
    
    livePullVC.config.shouldShowInAppPipIfAvailable = ^BOOL(BDLLivePullViewController * _Nonnull viewController, BDLActivityStatus status, BOOL isClose) {
        return NO;
    };
}

- (void)leaveLiveRoom:(NSDictionary *)arguments result:(FlutterResult)result {
    if (self.activity) {
        BDLLiveEngine *engine = [BDLLiveEngine sharedInstance];
        BDLLivePullViewController *livePullVC = [engine getLivePullViewController];
        
        if ([livePullVC isFloating]) {
            [livePullVC hideFloatingPlayerIfAvailable:NO];
        }
        else {
            [self.delegate hideLivePullViewController:livePullVC];
        }
        
        [engine leaveLiveRoom];
        self.activity = nil;
    }
    if (self.delegate) {
        [self.delegate destroy];
        self.delegate = nil;
    }
    result(nil);
}

- (void)setLiveRoomStatusListener:(NSDictionary *)arguments result:(FlutterResult)result {
    // The delegate has been set, so we can directly call result here.
    result(nil);
}

- (void)manualClickLiveRoomExitBtn:(NSDictionary *)arguments result:(FlutterResult)result {
    [[[BDLLiveEngine sharedInstance] getLivePullViewController] close:YES];
    result(nil);
}

- (void)openUrl:(NSDictionary *)arguments result:(FlutterResult)result {
    NSString *url = arguments[@"url"];
    BOOL enableFloating = [arguments[@"enableFloating"] boolValue];
    
    if (url) {
        [[[BDLLiveEngine sharedInstance] getLivePullViewController] defaultOpenURLAction:url enableFloating:enableFloating];
    }
    
    result(nil);
}


// MARK: - player
- (void)createPlayer:(NSDictionary *)arguments result:(FlutterResult)result {
    int64_t insId = [arguments[@"managerId"] longLongValue];
    BdliveFlutterViewerPlayer *player = [[BdliveFlutterViewerPlayer alloc] initWithRegistrar:self.registrar insId:insId];
    if (!self.playerMap) {
        self.playerMap = [NSMutableDictionary dictionary];
    }
    BDLActivity *activity = [[BDLActivity alloc] init];
    
    activity.activityId = arguments[@"activityId"];
    activity.token = arguments[@"token"];
    activity.authMode = [arguments[@"authMode"] integerValue];
    activity.isPortrait = [arguments[@"isPortrait"] boolValue];
    player.activity = activity;
    
    self.playerMap[@(insId)] = player;
    __weak typeof(self) weakSelf = self;
    player.destroyBlock = ^(BdliveFlutterViewerPlayer * _Nonnull player) {
        typeof(self) self = weakSelf;
        [self.playerMap removeObjectForKey:@(insId)];
    };
    result(nil);
}

@end
