//
//  BdliveFlutterViewerDelegate.m
// 
//   BDLive SDK License
//   
//   Copyright 2024 Beijing Volcano Engine Technology Ltd. All Rights Reserved.
//   
//   The BDLive SDK was developed by Beijing Volcanoengine Technology Ltd. (hereinafter “Volcano Engine”). 
//   Any copyright or patent right is owned by and proprietary material of the Volcano Engine. 
//   
//   BDLive SDK is available under the VolcLive product and licensed under the commercial license. 
//   Customers can contact service@volcengine.com for commercial licensing options. 
//   Here is also a link to subscription services agreement: https://www.volcengine.com/docs/6256/68938.
//   
//   Without Volcanoengine's prior written permission, any use of BDLive SDK, in particular any use for commercial purposes, is prohibited. 
//   This includes, without limitation, incorporation in a commercial product, use in a commercial service, or production of other artefacts for commercial purposes. 
//   
//   Without Volcanoengine's prior written permission, the BDLive SDK may not be reproduced, modified and/or made available in any form to any third party. 
//

#import "BdliveFlutterViewerDelegate.h"
#import <BDLive/BDLResponder.h>

@implementation BdliveFlutterViewerDelegate

- (void)openUrl:(NSString *)url enableFloating:(BOOL)enableFloating type:(BDLPageAdvertisementType)type {
    if (!url || ![url isKindOfClass:[NSString class]] || url.length == 0) {
        return;
    }
    [self emitEvent:@{
        @"url" : url ?: @"",
        @"enableFloating" : @(enableFloating),
        @"type" : @(type),
    } methodName:@"onOpenUrlCallback"];
}

- (void)showLivePullViewController:(BDLLivePullViewController *)livePullVC {
    livePullVC.modalPresentationStyle = UIModalPresentationFullScreen;
    [[BDLResponder topViewController] presentViewController:livePullVC animated:YES completion:nil];
}

- (void)hideLivePullViewController:(BDLLivePullViewController *)livePullVC {
    if (livePullVC.presentingViewController) {
        [livePullVC dismissViewControllerAnimated:YES completion:nil];
    }
}

- (void)livePullViewControllerDidAppear:(BDLLivePullViewController *)livePullVC {
    [self emitEvent:@{
        @"activity" : [self.activity yy_modelToJSONObject]
    } methodName:@"onLiveRoomActivityResume"];
}

- (void)livePullViewControllerDidDisappear:(BDLLivePullViewController *)livePullVC {
    [self emitEvent:@{
        @"activity" : [self.activity yy_modelToJSONObject]
    } methodName:@"onLiveRoomActivityPause"];
}

- (void)livePullViewControllerWillDestroy:(BDLLivePullViewController *)livePullVC {
    [self emitEvent:@{
        @"activity" : [self.activity yy_modelToJSONObject]
    } methodName:@"onLiveRoomFullDestroyed"];
}

- (void)livePullViewController:(BDLLivePullViewController *)livePullVC didClickHeaderAd:(NSString *)url floatingEnable:(BOOL)floatingEnable {
    [self openUrl:url enableFloating:floatingEnable type:BDLPageAdvertisementTypePageHeaderAd];
}

@end
