import 'dart:collection';

import 'package:bdlive_flutter_viewer/src/bdlive_viewer_manager_impl.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/material.dart';
import '../api/bdlive_flutter_viewer_api.dart';
import '../api/bdlive_model.dart';
import 'base/bdlive_event_channel.dart';
import 'bdlive_room_status_listener_extension.dart';

class BdliveFlutterViewerImpl implements BdliveViewerApi {

  RTCEventChannel? _observerChannel;

  @override
  Future<bool> enterLiveRoom(BdlActivityConfig activity) async {
    var result = await _invokeMethod<bool>(
        'enterLiveRoom', activity.toMap());
    return result ?? false;
  }

  @override
  Future<void> leaveLiveRoom() {
    return _invokeMethod('leaveLiveRoom');
  }

  @override
  Future<void> setLiveRoomStatusListener(
      BdlLiveRoomStatusListener? listener) async {
    _observerChannel?.cancel();
    if (listener == null) {
      return;
    }
    var result = await _invokeMethod("setLiveRoomStatusListener");
    RTCEventChannel channel =
        RTCEventChannel('com.bytedance.flutter_viewer.listener.event');
    channel.listen((String methodName, Map<dynamic, dynamic> dic) {
      debugPrint('dart - listen = $dic  $listener');

      listener.process(methodName, dic);
    });
    _observerChannel = channel;
    return result;
  }

  @override
  Future<void> manualClickLiveRoomExitBtn() {
    return _invokeMethod('manualClickLiveRoomExitBtn');
  }

  @override
  Future<void> openUrl(
      String url, BdlViewerOpenUrlType type, bool enableFloating) {
    HashMap<String, dynamic> dic = HashMap();
    dic['url'] = url;
    dic['type'] = type.index + 1;
    dic['enableFloating'] = enableFloating;
    return _invokeMethod('openUrl', dic);
  }

  Future<T?> _invokeMethod<T>(String method, [ dynamic arguments ]) {
    return BdliveViewerManagerImpl.staticChannel.invokeMethod(method, arguments);
  }
}
