


import 'package:bdlive_flutter_viewer/src/base/bdlive_enum_convert.dart';
import 'package:flutter/material.dart';
import '../api/bdlive_viewer_player_delegate.dart';
import '../api/bdlive_model.dart';

extension BdliveViewerPlayerDelegateExtension on BdliveViewerPlayerDelegate {
  void process(String methodName, Map<dynamic, dynamic> dic) {
    switch (methodName) {
      case 'liveRoomStatusChanged':
        BdLiveRoomStatus status = (dic['status'] as int).liveRoomStatus;
        liveRoomStatusChanged?.call(status);
        break;
      case 'playableStatusChanged':
        BdlivePlayableStatus status = (dic['playableStatus'] as int).playableStatus;
        playableStatusChanged?.call(status);
        break;
      case 'isPlayingChanged':
        bool isPlaying = dic['isPlaying'];
        isPlayingChanged?.call(isPlaying);
        break;
      case 'videoSizeChanged':
        int width = dic['width'];
        int height = dic['height'];
        videoSizeChanged?.call(width, height);
        break;
      case 'isStallingChanged':
        bool isStalling = dic['isStalling'];
        isStallingChanged?.call(isStalling);
        break;
      case 'vodErrorOccurred':
        BdliveError error = BdliveError.fromMap(dic['error']);
        vodErrorOccurred?.call(error);
        break;
      case 'liveErrorOccurred':
        BdliveError error = BdliveError.fromMap(dic['error']);
        liveErrorOccurred?.call(error);
        break;
      case 'playErrorStatusChanged':
        bool isPlayError = dic['isPlayError'];
        playErrorStatusChanged?.call(isPlayError);
        break;
      case 'vodPrepared':
        vodPrepared?.call();
        break;
      case 'vodRenderStarted':
        vodRenderStarted?.call();
        break;
      case 'vodAutoSeekPreviousTime':
        int seekTimeInMills = dic['seekTimeInMills'];
        vodAutoSeekPreviousTime?.call(seekTimeInMills);
        break;
      case 'vodCurPlayTimeChanged':
        int curTimeInMills = dic['curTimeInMills'];
        vodCurPlayTimeChanged?.call(curTimeInMills);
        break;
      case 'vodDurationChanged':
        int durationInMills = dic['durationInMills'];
        vodDurationChanged?.call(durationInMills);
        break;
      case 'livePrepared':
        livePrepared?.call();
        break;
      case 'liveFirstFrameRendered':
        bool isFirstFrame = dic['isFirstFrame'];
        liveFirstFrameRendered?.call(isFirstFrame);
        break;
      case 'coverImageVisibleChanged':
        bool isVisible = dic['isVisible'];
        coverImageVisibleChanged?.call(isVisible);
        break;
      case 'resolutionInfoChanged':
        List<int> resolutions = dic['resolutions'].cast<int>();
        int defaultResolution = dic['defaultResolution'];
        var list = resolutions.map((e) => e.videoResolution).toList();
        resolutionInfoChanged?.call(list, defaultResolution.videoResolution);
        break;
      case 'coverImageSizeChanged':
        int width = dic['width'];
        int height = dic['height'];
        coverImageSizeChanged?.call(width, height);
        break;
      case 'onUserForceOffline':
        BdliveForceOfflineReason reason = (dic['reason'] as int).forceOfflineReason;
        onUserForceOffline?.call(reason);
        break;
      case 'onRelease':
        onRelease?.call();
        break;
      default:
        break;
    }
  }
}